@@ -1,10 +1,6 @@
 use git_config::file::GitConfig;
-use git_features::progress::Progress;
-use git_hash::bstr::ByteSlice;
-use std::{
-    convert::TryFrom,
-    path::{Path, PathBuf},
-};
+use git_repository::{object::bstr::ByteSlice, progress, Progress};
+use std::path::{Path, PathBuf};
 
 #[derive(Copy, Clone, Eq, PartialEq)]
 pub enum Mode {
@@ -18,37 +14,41 @@ impl Default for Mode {
     }
 }
 
-fn find_git_repository_workdirs<P: Progress>(root: impl AsRef<Path>, mut progress: P) -> impl Iterator<Item = PathBuf>
+enum RepoKind {
+    Bare,
+    WorkingTree,
+}
+
+fn find_git_repository_workdirs<P: Progress>(
+    root: impl AsRef<Path>,
+    mut progress: P,
+) -> impl Iterator<Item = (PathBuf, RepoKind)>
 where
     <P as Progress>::SubProgress: Sync,
 {
-    progress.init(None, git_features::progress::count("filesystem items"));
-    fn is_repository(path: &Path) -> bool {
+    progress.init(None, progress::count("filesystem items"));
+    fn is_repository(path: &Path) -> Option<git_repository::Kind> {
         if !(path.is_dir() && path.ends_with(".git")) {
-            return false;
+            return None;
         }
-        path.join("HEAD").is_file() && path.join("config").is_file()
-    }
-    fn into_workdir(path: PathBuf) -> PathBuf {
-        fn is_bare(path: &Path) -> bool {
-            !path.join("index").exists()
-        }
-        if is_bare(&path) {
-            path
+        if path.join("HEAD").is_file() && path.join("config").is_file() {
+            git_repository::path::is_git(path).ok()
         } else {
-            path.parent().expect("git is never in the root").to_owned()
+            None
+        }
+    }
+    fn into_workdir(git_dir: PathBuf) -> PathBuf {
+        if git_repository::path::is_bare(&git_dir) {
+            git_dir
+        } else {
+            git_dir.parent().expect("git is never in the root").to_owned()
         }
     }
 
-    #[derive(Debug)]
+    #[derive(Debug, Default)]
     struct State {
         is_repo: bool,
-    }
-
-    impl Default for State {
-        fn default() -> Self {
-            State { is_repo: false }
-        }
+        is_bare: bool,
     }
 
     let walk = jwalk::WalkDirGeneric::<((), State)>::new(root)
@@ -63,39 +63,51 @@ where
     #[cfg(all(target_os = "macos", target_arch = "aarch64"))]
     let walk = walk.parallelism(jwalk::Parallelism::RayonNewPool(4));
 
-    walk.process_read_dir(move |_depth, _path, _read_dir_state, children| {
-        let mut found_repo = false;
-        for entry in children.iter_mut().flatten() {
-            if is_repository(&entry.path()) {
-                entry.client_state = State { is_repo: true };
+    walk.process_read_dir(move |_depth, _path, _read_dir_state, siblings| {
+        let mut found_any_repo = false;
+        let mut found_bare_repo = false;
+        for entry in siblings.iter_mut().flatten() {
+            let path = entry.path();
+            if let Some(kind) = is_repository(&path) {
+                let is_bare = kind.is_bare();
+                entry.client_state = State { is_repo: true, is_bare };
                 entry.read_children_path = None;
-                found_repo = true;
-                break;
+
+                found_any_repo = !is_bare;
+                found_bare_repo = is_bare;
             }
         }
-        if found_repo {
-            children.retain(|e| e.as_ref().map(|e| e.client_state.is_repo).unwrap_or(false));
+        // Only return paths which are repositories are further participating in the traversal
+        // Don't let bare repositories cause siblings to be pruned.
+        if found_any_repo && !found_bare_repo {
+            siblings.retain(|e| e.as_ref().map(|e| e.client_state.is_repo).unwrap_or(false));
         }
     })
     .into_iter()
     .inspect(move |_| progress.inc())
     .filter_map(Result::ok)
     .filter(|e| e.client_state.is_repo)
-    .map(|e| into_workdir(e.path()))
+    .map(|e| {
+        (
+            into_workdir(e.path()),
+            if e.client_state.is_bare {
+                RepoKind::Bare
+            } else {
+                RepoKind::WorkingTree
+            },
+        )
+    })
 }
 
 fn find_origin_remote(repo: &Path) -> anyhow::Result<Option<git_url::Url>> {
-    let config_bytes;
-    let config = {
-        let non_bare = repo.join(".git").join("config");
-        config_bytes = std::fs::read(non_bare).or_else(|_| std::fs::read(repo.join("config")))?;
-        GitConfig::try_from(&config_bytes).map_err(|e| e.to_owned())?
-    };
+    let non_bare = repo.join(".git").join("config");
+    let config = GitConfig::open(non_bare.as_path()).or_else(|_| GitConfig::open(repo.join("config").as_path()))?;
     Ok(config.value("remote", Some("origin"), "url").ok())
 }
 
 fn handle(
     mode: Mode,
+    kind: RepoKind,
     git_workdir: &Path,
     canonicalized_destination: &Path,
     progress: &mut impl Progress,
@@ -159,7 +171,16 @@ fn handle(
                 .as_ref()
                 .ok_or_else(|| anyhow::Error::msg(format!("Remote URLs must have host names: {}", url)))?,
         )
-        .join(to_relative(git_url::expand_path(None, url.path.as_bstr())?));
+        .join(to_relative({
+            let mut path = git_url::expand_path(None, url.path.as_bstr())?;
+            match kind {
+                RepoKind::Bare => path,
+                RepoKind::WorkingTree => {
+                    path.set_extension("");
+                    path
+                }
+            }
+        }));
 
     if let Ok(destination) = destination.canonicalize() {
         if git_workdir.canonicalize()? == destination {
@@ -190,7 +211,7 @@ pub fn discover<P: Progress>(
 where
     <<P as Progress>::SubProgress as Progress>::SubProgress: Sync,
 {
-    for git_workdir in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
+    for (git_workdir, _kind) in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
         writeln!(&mut out, "{}", git_workdir.display())?;
     }
     Ok(())
@@ -207,8 +228,8 @@ where
 {
     let mut num_errors = 0usize;
     let destination = destination.as_ref().canonicalize()?;
-    for path_to_move in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
-        if let Err(err) = handle(mode, &path_to_move, &destination, &mut progress) {
+    for (path_to_move, kind) in find_git_repository_workdirs(source_dir, progress.add_child("Searching repositories")) {
+        if let Err(err) = handle(mode, kind, &path_to_move, &destination, &mut progress) {
             progress.fail(format!(
                 "Error when handling directory {:?}: {}",
                 path_to_move.display(),
