@@ -1,10 +1,20 @@
-use std::{fs, io::Read, path::Path};
+use std::{
+    fs,
+    io::Read,
+    path::Path,
+    sync::{atomic::AtomicBool, Arc},
+};
 
 use anyhow::{anyhow, Result};
 use quick_error::quick_error;
 
-use git_features::progress::{self, Progress};
-use git_odb::{loose, pack, Write};
+use git_repository::{
+    hash,
+    hash::ObjectId,
+    object, odb,
+    odb::{loose, pack, Write},
+    progress, Progress,
+};
 
 #[derive(PartialEq, Debug)]
 pub enum SafetyCheck {
@@ -69,27 +79,27 @@ quick_error! {
             source(err)
             from()
         }
-        OdbWrite(err: loose::db::write::Error) {
+        OdbWrite(err: loose::write::Error) {
             display("An object could not be written to the database")
             source(err)
             from()
         }
-        Write(err: Box<dyn std::error::Error + Send + Sync>, kind: git_object::Kind, id: git_hash::ObjectId) {
+        Write(err: Box<dyn std::error::Error + Send + Sync>, kind: object::Kind, id: ObjectId) {
             display("Failed to write {} object {}", kind, id)
             source(&**err)
         }
-        Verify(err: git_odb::data::verify::Error) {
+        Verify(err: pack::data::object::verify::Error) {
             display("Object didn't verify after right after writing it")
             source(err)
             from()
         }
-        ObjectEncodeMismatch(kind: git_object::Kind, actual: git_hash::ObjectId, expected: git_hash::ObjectId) {
+        ObjectEncodeMismatch(kind: object::Kind, actual: ObjectId, expected: ObjectId) {
             display("{} object {} wasn't re-encoded without change - new hash is {}", kind, expected, actual)
         }
-        WrittenFileMissing(id: git_hash::ObjectId) {
+        WrittenFileMissing(id: ObjectId) {
             display("The recently written file for loose object {} could not be found", id)
         }
-        WrittenFileCorrupt(err: loose::db::find::Error, id: git_hash::ObjectId) {
+        WrittenFileCorrupt(err: loose::find::Error, id: ObjectId) {
             display("The recently written file for loose object {} cold not be read", id)
             source(err)
         }
@@ -98,19 +108,14 @@ quick_error! {
 
 #[allow(clippy::large_enum_variant)]
 enum OutputWriter {
-    Loose(loose::Db),
-    Sink(git_odb::Sink),
+    Loose(loose::Store),
+    Sink(odb::Sink),
 }
 
-impl git_odb::write::Write for OutputWriter {
+impl git_repository::odb::Write for OutputWriter {
     type Error = Error;
 
-    fn write_buf(
-        &self,
-        kind: git_object::Kind,
-        from: &[u8],
-        hash: git_hash::Kind,
-    ) -> Result<git_hash::ObjectId, Self::Error> {
+    fn write_buf(&self, kind: object::Kind, from: &[u8], hash: hash::Kind) -> Result<ObjectId, Self::Error> {
         match self {
             OutputWriter::Loose(db) => db.write_buf(kind, from, hash).map_err(Into::into),
             OutputWriter::Sink(db) => db.write_buf(kind, from, hash).map_err(Into::into),
@@ -119,11 +124,11 @@ impl git_odb::write::Write for OutputWriter {
 
     fn write_stream(
         &self,
-        kind: git_object::Kind,
+        kind: object::Kind,
         size: u64,
         from: impl Read,
-        hash: git_hash::Kind,
-    ) -> Result<git_hash::ObjectId, Self::Error> {
+        hash: hash::Kind,
+    ) -> Result<ObjectId, Self::Error> {
         match self {
             OutputWriter::Loose(db) => db.write_stream(kind, size, from, hash).map_err(Into::into),
             OutputWriter::Sink(db) => db.write_stream(kind, size, from, hash).map_err(Into::into),
@@ -134,8 +139,8 @@ impl git_odb::write::Write for OutputWriter {
 impl OutputWriter {
     fn new(path: Option<impl AsRef<Path>>, compress: bool) -> Self {
         match path {
-            Some(path) => OutputWriter::Loose(loose::Db::at(path.as_ref())),
-            None => OutputWriter::Sink(git_odb::sink().compress(compress)),
+            Some(path) => OutputWriter::Loose(loose::Store::at(path.as_ref())),
+            None => OutputWriter::Sink(odb::sink().compress(compress)),
         }
     }
 }
@@ -146,6 +151,7 @@ pub struct Context {
     pub delete_pack: bool,
     pub sink_compress: bool,
     pub verify: bool,
+    pub should_interrupt: Arc<AtomicBool>,
 }
 
 pub fn pack_or_pack_index(
@@ -158,6 +164,7 @@ pub fn pack_or_pack_index(
         delete_pack,
         sink_compress,
         verify,
+        should_interrupt,
     }: Context,
 ) -> Result<()> {
     use anyhow::Context;
@@ -198,17 +205,17 @@ pub fn pack_or_pack_index(
             move || {
                 let out = OutputWriter::new(object_path.clone(), sink_compress);
                 let object_verifier = if verify {
-                    object_path.as_ref().map(loose::Db::at)
+                    object_path.as_ref().map(loose::Store::at)
                 } else {
                     None
                 };
                 let mut read_buf = Vec::new();
                 move |object_kind, buf, index_entry, progress| {
                     let written_id = out
-                        .write_buf(object_kind, buf, git_hash::Kind::Sha1)
+                        .write_buf(object_kind, buf, hash::Kind::Sha1)
                         .map_err(|err| Error::Write(Box::new(err) as Box<dyn std::error::Error + Send + Sync>, object_kind, index_entry.oid))?;
                     if written_id != index_entry.oid {
-                        if let git_object::Kind::Tree = object_kind {
+                        if let object::Kind::Tree = object_kind {
                             progress.info(format!("The tree in pack named {} was written as {} due to modes 100664 and 100640 rewritten as 100644.", index_entry.oid, written_id));
                         } else {
                             return Err(Error::ObjectEncodeMismatch(object_kind, index_entry.oid, written_id));
@@ -229,6 +236,7 @@ pub fn pack_or_pack_index(
             algorithm,
             thread_limit,
             check: check.into(),
+            should_interrupt
         },
     ).map(|(_, _, c)| progress::DoOrDiscard::from(c)).with_context(|| "Failed to explode the entire pack - some loose objects may have been created nonetheless")?;
 
