@@ -1,6 +1,6 @@
 use crate::OutputFormat;
-use git_features::progress::Progress;
-use git_odb::pack;
+use git_repository::{odb::pack, Progress};
+use std::sync::atomic::AtomicBool;
 use std::{fs, io, path::PathBuf, str::FromStr};
 
 #[derive(PartialEq, Debug)]
@@ -48,10 +48,11 @@ impl From<IterationMode> for pack::data::input::Mode {
     }
 }
 
-pub struct Context<W: io::Write> {
+pub struct Context<'a, W: io::Write> {
     pub thread_limit: Option<usize>,
     pub iteration_mode: IterationMode,
     pub format: OutputFormat,
+    pub should_interrupt: &'a AtomicBool,
     pub out: W,
 }
 
@@ -67,11 +68,16 @@ pub fn stream_len(mut s: impl io::Seek) -> io::Result<u64> {
 
 pub const PROGRESS_RANGE: std::ops::RangeInclusive<u8> = 2..=3;
 
+pub enum PathOrRead {
+    Path(PathBuf),
+    Read(Box<dyn std::io::Read + Send + 'static>),
+}
+
 pub fn from_pack(
-    pack: Option<PathBuf>,
+    pack: PathOrRead,
     directory: Option<PathBuf>,
     progress: impl Progress,
-    ctx: Context<impl io::Write>,
+    ctx: Context<'static, impl io::Write>,
 ) -> anyhow::Result<()> {
     use anyhow::Context;
     let options = pack::bundle::write::Options {
@@ -82,15 +88,28 @@ pub fn from_pack(
     let out = ctx.out;
     let format = ctx.format;
     let res = match pack {
-        Some(pack) => {
+        PathOrRead::Path(pack) => {
             let pack_len = pack.metadata()?.len();
             let pack_file = fs::File::open(pack)?;
-            pack::Bundle::write_to_directory_eagerly(pack_file, Some(pack_len), directory, progress, options)
-        }
-        None => {
-            let stdin = io::stdin();
-            pack::Bundle::write_to_directory_eagerly(stdin, None, directory, progress, options)
+            pack::Bundle::write_to_directory_eagerly(
+                pack_file,
+                Some(pack_len),
+                directory,
+                progress,
+                ctx.should_interrupt,
+                None,
+                options,
+            )
         }
+        PathOrRead::Read(input) => pack::Bundle::write_to_directory_eagerly(
+            input,
+            None,
+            directory,
+            progress,
+            ctx.should_interrupt,
+            None,
+            options,
+        ),
     }
     .with_context(|| "Failed to write pack and index")?;
     match format {
